"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapWindowToElement = exports.mapElementToScreenRect = exports.estimateWindowBorder = exports.estimateBrowserZoom = void 0;

var BrowserPopupWindow_1 = require("./BrowserPopupWindow");

function estimateBrowserZoom(_window) {
  // one of them might be off by a lot due to developer console or other browser plugin
  var _BrowserPopupWindow_ = BrowserPopupWindow_1.gerWindowBorder(),
      _BrowserPopupWindow_2 = (0, _slicedToArray2.default)(_BrowserPopupWindow_, 3),
      topBorder = _BrowserPopupWindow_2[0],
      sideBorder = _BrowserPopupWindow_2[1],
      bottomBorder = _BrowserPopupWindow_2[2];

  if (_window.outerWidth === _window.screen.availWidth) {
    sideBorder = 0;
    bottomBorder = 0;
  }

  var xRatio = (_window.outerWidth - sideBorder * 2) / _window.innerWidth;
  var yRatio = (_window.outerHeight - topBorder - bottomBorder) / _window.innerHeight;
  var zoomRatio = Math.min(yRatio, xRatio);

  if (zoomRatio > 1.8) {
    zoomRatio = Math.round(zoomRatio);
  } else if (zoomRatio > 0.73) {
    zoomRatio = Math.round(zoomRatio * 20) / 20;
  } else {
    zoomRatio = 2 / Math.round(2 / zoomRatio);
  }

  console.log("zoom ".concat(zoomRatio));
  return zoomRatio;
}

exports.estimateBrowserZoom = estimateBrowserZoom;

function estimateWindowBorder(_window) {
  var addBorder = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var zoom = _window ? estimateBrowserZoom(_window) : 1;
  var xBorder = _window.outerWidth - _window.innerWidth * zoom >> 1;
  var yBorder = Math.round(_window.outerHeight - _window.innerHeight * zoom);

  if (xBorder > 32) {
    // probably because of debugger console, assume it's in the right side
    xBorder = 8;
  } else {
    yBorder -= xBorder;
  }

  return [xBorder, yBorder, zoom];
}

exports.estimateWindowBorder = estimateWindowBorder;

var MapRect2D = /*#__PURE__*/function () {
  function MapRect2D() {
    (0, _classCallCheck2.default)(this, MapRect2D);
  }

  (0, _createClass2.default)(MapRect2D, [{
    key: "init",
    value: function init(x1, y1, w1, h1, x2, y2, w2, h2) {
      this.scaleX = w2 / w1;
      this.scaleY = h2 / h1;
      this.offsetX = x2 - x1 * this.scaleX;
      this.offsetY = y2 - y1 * this.scaleY;
    }
  }, {
    key: "map",
    value: function map(pt) {
      return {
        x: pt.x * this.scaleX + this.offsetX,
        y: pt.y * this.scaleY + this.offsetY
      };
    }
  }, {
    key: "revertMap",
    value: function revertMap(pt) {
      return {
        x: (pt.x - this.offsetX) / this.scaleX,
        y: (pt.y - this.offsetY) / this.scaleY
      };
    }
  }]);
  return MapRect2D;
}();

function mapElementToScreenRect(element, rect) {
  if (!element) {
    return null;
  }

  var clientRect = element.getBoundingClientRect();
  var mapRect = new MapRect2D();
  mapRect.init(0, 0, element.offsetWidth, element.offsetHeight, clientRect.x, clientRect.y, clientRect.width, clientRect.height);
  var mappedRect;

  if (rect) {
    var _mapRect$map = mapRect.map({
      x: rect.left,
      y: rect.top
    }),
        x = _mapRect$map.x,
        y = _mapRect$map.y;

    var _mapRect$map2 = mapRect.map({
      x: rect.left + rect.width,
      y: rect.top + rect.height
    }),
        x2 = _mapRect$map2.x,
        y2 = _mapRect$map2.y;

    mappedRect = {
      left: x,
      top: y,
      width: x2 - x,
      height: y2 - y
    };
  } else {
    mappedRect = {
      left: clientRect.left,
      top: clientRect.top,
      width: clientRect.width,
      height: clientRect.height
    };
  }

  var _document = element.ownerDocument;
  var _window = _document.defaultView;

  if (!_window) {
    return clientRect;
  } // recursively get rect if it's an iframe


  if (_window.frameElement) {
    return mapElementToScreenRect(_window.frameElement, mappedRect);
  }

  var _estimateWindowBorder = estimateWindowBorder(_window),
      _estimateWindowBorder2 = (0, _slicedToArray2.default)(_estimateWindowBorder, 3),
      xBorder = _estimateWindowBorder2[0],
      yBorder = _estimateWindowBorder2[1],
      zoom = _estimateWindowBorder2[2];

  if (zoom !== 1) {
    mappedRect.left *= zoom;
    mappedRect.top *= zoom;
    mappedRect.width *= zoom;
    mappedRect.height *= zoom;
  }

  mappedRect.left += _window.screenX + xBorder;
  mappedRect.top += _window.screenY + yBorder;
  return mappedRect;
}

exports.mapElementToScreenRect = mapElementToScreenRect;

function mapWindowToElement(targetElement, fromWindow, fromRect) {
  var removeBorder = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

  if (!targetElement) {
    return null;
  }

  if (fromWindow) {
    fromRect = {
      left: fromWindow.screenX,
      top: fromWindow.screenY,
      width: fromWindow.outerWidth,
      height: fromWindow.outerHeight
    };

    if (removeBorder) {
      var _BrowserPopupWindow_3 = (0, _slicedToArray2.default)(BrowserPopupWindow_1.popupWindowBorder, 3),
          topBorder = _BrowserPopupWindow_3[0],
          sideBorder = _BrowserPopupWindow_3[1],
          bottomBorder = _BrowserPopupWindow_3[2];

      fromRect.left += sideBorder;
      fromRect.top += topBorder;
      fromRect.width -= sideBorder * 2;
      fromRect.height -= topBorder + bottomBorder;
    }
  } else if (!fromRect) {
    return null;
  }

  var _document = targetElement.ownerDocument;
  var _window = _document.defaultView;

  if (!_window) {
    return fromRect;
  } // recursively get rect if it's an iframe


  if (_window.frameElement) {
    fromRect = mapWindowToElement(_window.frameElement, null, fromRect);
  } else {
    var _estimateWindowBorder3 = estimateWindowBorder(_window),
        _estimateWindowBorder4 = (0, _slicedToArray2.default)(_estimateWindowBorder3, 3),
        xBorder = _estimateWindowBorder4[0],
        yBorder = _estimateWindowBorder4[1],
        zoom = _estimateWindowBorder4[2];

    fromRect.left -= _window.screenX + xBorder;
    fromRect.top -= _window.screenY + yBorder;

    if (zoom !== 1) {
      fromRect.left /= zoom;
      fromRect.top /= zoom;
      fromRect.width /= zoom;
      fromRect.height /= zoom;
    }
  }

  var clientRect = targetElement.getBoundingClientRect();
  var mapRect = new MapRect2D();
  mapRect.init(0, 0, targetElement.offsetWidth, targetElement.offsetHeight, clientRect.x, clientRect.y, clientRect.width, clientRect.height);
  var mappedRect;

  var _mapRect$revertMap = mapRect.revertMap({
    x: fromRect.left,
    y: fromRect.top
  }),
      x = _mapRect$revertMap.x,
      y = _mapRect$revertMap.y;

  var _mapRect$revertMap2 = mapRect.revertMap({
    x: fromRect.left + fromRect.width,
    y: fromRect.top + fromRect.height
  }),
      x2 = _mapRect$revertMap2.x,
      y2 = _mapRect$revertMap2.y;

  mappedRect = {
    left: x,
    top: y,
    width: x2 - x,
    height: y2 - y
  };
  return mappedRect;
}

exports.mapWindowToElement = mapWindowToElement;