"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var __importDefault = void 0 && (void 0).__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

var react_1 = __importDefault(require("react"));

var react_dom_1 = __importDefault(require("react-dom"));

var debounce_1 = __importDefault(require("lodash/debounce"));

var BrowserPopupWindow_1 = require("./BrowserPopupWindow");

var onNewWindowResize = debounce_1.default(function () {
  // add/remove element on main document, force it to dispatch resize observer event on the popup window
  var div = document.createElement('div');
  document.body.append(div);
  div.remove(); // TODO update resize event
}, 200);
/**
 * The NewWindow class object.
 * @public
 */

var NewWindow = /*#__PURE__*/function (_react_1$default$Pure) {
  (0, _inherits2.default)(NewWindow, _react_1$default$Pure);

  var _super = (0, _createSuper2.default)(NewWindow);

  /**
   * The NewWindow function constructor.
   * @param {Object} props
   */
  function NewWindow(props) {
    var _this;

    (0, _classCallCheck2.default)(this, NewWindow);
    _this = _super.call(this, props);
    _this.released = false;
    _this.container = document.createElement('div');
    _this.state = {
      mounted: false
    };

    _this.onMainWindowUnload = function () {
      if (_this.window) {
        _this.window.close();
      }
    };
    /**
     * Release the new window and anything that was bound to it.
     */


    _this.release = function (event) {
      // This method can be called once.
      if (_this.released) {
        return;
      }

      _this.released = true;
      window.removeEventListener('beforeunload', _this.onMainWindowUnload);

      _this.window.addEventListener('beforeunload', _this.release);

      if (event) {
        // Call any function bound to the `onUnload` prop.
        var onClose = _this.props.onClose;

        if (typeof onClose === 'function') {
          onClose();
        }
      }
    };

    return _this;
  }
  /**
   * Render the NewWindow component.
   */


  (0, _createClass2.default)(NewWindow, [{
    key: "render",
    value: function render() {
      if (!this.state.mounted) return null;
      return react_dom_1.default.createPortal(this.props.children, this.container);
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.openChild();
      this.setState({
        mounted: true
      });
    }
    /**
     * Create the new window when NewWindow component mount.
     */

  }, {
    key: "openChild",
    value: function openChild() {
      var _this2 = this;

      var _this$props = this.props,
          url = _this$props.url,
          title = _this$props.title,
          name = _this$props.name,
          width = _this$props.width,
          height = _this$props.height,
          initPopupInnerRect = _this$props.initPopupInnerRect,
          initPopupOuterRect = _this$props.initPopupOuterRect,
          onBlock = _this$props.onBlock,
          onOpen = _this$props.onOpen;
      var features = {
        width: width,
        height: height
      };

      if (initPopupOuterRect) {
        features = initPopupOuterRect();

        var _BrowserPopupWindow_ = (0, _slicedToArray2.default)(BrowserPopupWindow_1.popupWindowBorder, 3),
            topBorder = _BrowserPopupWindow_[0],
            sideBorder = _BrowserPopupWindow_[1],
            bottomBorder = _BrowserPopupWindow_[2];

        if (!BrowserPopupWindow_1.isSafari) {
          features.width -= sideBorder * 2;
          features.height -= topBorder + bottomBorder;
        }
      } else if (initPopupInnerRect) {
        features = initPopupInnerRect();

        var _BrowserPopupWindow_2 = (0, _slicedToArray2.default)(BrowserPopupWindow_1.popupWindowBorder, 2),
            _topBorder = _BrowserPopupWindow_2[0],
            _sideBorder = _BrowserPopupWindow_2[1];

        features.left -= _sideBorder;
        features.top -= _topBorder;

        if (BrowserPopupWindow_1.isSafari) {
          features.height += _topBorder;
        }
      } else {
        features.left = window.top.outerWidth / 2 + window.top.screenX - width / 2;
        features.top = window.top.outerHeight / 2 + window.top.screenY - height / 2;
      } // Open a new window.


      this.window = window.open(url, name, toWindowFeatures(features)); // Check if the new window was successfully opened.

      if (this.window) {
        window.addEventListener('beforeunload', this.onMainWindowUnload);
        this.window.addEventListener('resize', onNewWindowResize);
        this.window.document.title = title || document.title;
        this.window.document.body.appendChild(this.container); // If specified, copy styles from parent window's document.

        if (this.props.copyStyles) {
          setTimeout(function () {
            return copyStyles(document, _this2.window.document);
          }, 0);
        }

        if (typeof onOpen === 'function') {
          onOpen(this.window);
        } // Release anything bound to this component before the new window unload.


        this.window.addEventListener('beforeunload', this.release);
      } else {
        // Handle error on opening of new window.
        if (typeof onBlock === 'function') {
          onBlock();
        } else {
          console.warn('A new window could not be opened. Maybe it was blocked.');
        }
      }
    }
    /**
     * Close the opened window (if any) when NewWindow will unmount.
     */

  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      if (this.window) {
        this.release();
        this.window.close();
      }
    }
  }]);
  return NewWindow;
}(react_1.default.PureComponent);

NewWindow.supported = BrowserPopupWindow_1.popupSupported;
/**
 * NewWindow default props.
 */

NewWindow.defaultProps = {
  url: '',
  name: '',
  width: 640,
  height: 480,
  copyStyles: true
};
/**
 * Utility functions.
 * @private
 */

/**
 * Copy styles from a source document to a target.
 * @param {Object} source
 * @param {Object} target
 * @private
 */

function copyStyles(source, target) {
  Array.from(source.styleSheets).forEach(function (styleSheet) {
    // For <style> elements
    var rules;

    if (styleSheet.href) {
      // for <link> elements loading CSS from a URL
      var newLinkEl = source.createElement('link');
      newLinkEl.rel = 'stylesheet';
      newLinkEl.href = styleSheet.href;
      target.head.appendChild(newLinkEl);
    } else {
      try {
        rules = styleSheet.cssRules;
      } catch (err) {// can't access crossdomain rules
      }

      if (rules) {
        var newStyleEl = source.createElement('style'); // Write the text of each rule into the body of the style element

        Array.from(styleSheet.cssRules).forEach(function (cssRule) {
          var cssText = cssRule.cssText,
              type = cssRule.type;
          var returnText = cssText; // Check if the cssRule type is CSSImportRule (3) or CSSFontFaceRule (5) to handle local imports on a about:blank page
          // '/custom.css' turns to 'http://my-site.com/custom.css'

          if ([3, 5].includes(type)) {
            returnText = cssText.split('url(').map(function (line) {
              if (line[1] === '/') {
                return "".concat(line.slice(0, 1)).concat(window.location.origin).concat(line.slice(1));
              }

              return line;
            }).join('url(');
          }

          newStyleEl.appendChild(source.createTextNode(returnText));
        });
        target.head.appendChild(newStyleEl);
      }
    }
  });
}
/**
 * Convert features props to window features format (name=value,other=value).
 * @param {Object} obj
 * @return {String}
 * @private
 */


function toWindowFeatures(obj) {
  return Object.keys(obj).reduce(function (features, name) {
    var value = obj[name];

    if (typeof value === 'boolean') {
      features.push("".concat(name, "=").concat(value ? 'yes' : 'no'));
    } else {
      features.push("".concat(name, "=").concat(value));
    }

    return features;
  }, []).join(',');
}
/**
 * Component export.
 * @private
 */


exports.default = NewWindow;